package org.vraptor.http;

import java.io.IOException;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.vraptor.LogicException;
import org.vraptor.component.ComponentNotFoundException;
import org.vraptor.component.LogicNotFoundException;
import org.vraptor.core.Controller;
import org.vraptor.core.WebRequest;
import org.vraptor.interceptor.InterceptorInstantiationException;
import org.vraptor.url.InvalidURLException;
import org.vraptor.view.ViewException;

/**
 *
 *	The Dispatcher is responsible for translating between the Servlet API and VRaptor.
 * 
 * @author Rafael Ferreira
 */
public class DefaultDispatcher implements Dispatcher {
	private static final Logger LOG = Logger.getLogger(DefaultDispatcher.class);
	
	private final static String CHECK_THE_LOGS_MESAGE = " For further information, check the log files for the root cause."
		+ " Enable VRaptor DEBUG/TRACE messages in log4J if necessary.";

	private final Controller controller;
	
	public DefaultDispatcher(Controller controller) {
		this.controller = controller;
	}

	public void dispatch(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, ServletContext servletContext) throws IOException, ServletException {
		VRaptorServletRequest request = new VRaptorServletRequest(httpServletRequest);
		VRaptorServletResponse response = new VRaptorServletResponse(httpServletResponse);
		WebRequest webRequest = new WebRequest(request, response, servletContext);
		
		try {
			controller.execute(webRequest);
		} catch (LogicNotFoundException e) {
			LOG.debug("Logic not found", e);
			sendError(webRequest.getResponse(), 404, "URL not found, no logic matches for this component.", e);
		} catch (InvalidURLException ex) {
			throw new ServletException("Unable to deal with selected url", ex);
		} catch (ComponentNotFoundException e) {
			LOG.debug("Component not found", e);
			sendError(webRequest.getResponse(), 404, "URL not found, no registered component matches.", e);
		} catch (InterceptorInstantiationException e) {
			LOG.debug("Instatiating an interceptor resulted an exception.", e);
			throw new ServletException(e.getMessage(), e);
		} catch (ViewException ex) {
			LOG.debug("Rendering view resulted an exception.", ex);
			if (ex.getCause() == null) {
				throw new ServletException(ex.getMessage(), ex);
			} else {
				throw new ServletException(ex.getCause().getMessage(), ex.getCause());
			}
		} catch (LogicException ex) {
			LOG.debug("Logic exception.", ex);
	
			if (ex.getCause() == null) {
				throw new ServletException(ex.getMessage(), ex);
			} else {
				throw new ServletException(ex.getCause().getMessage(), ex.getCause());
			}
		} catch (Exception e) {
			LOG.debug("VRaptor unexpected exception", e);
			throw new ServletException(e);
		}
	}

	private void sendError(VRaptorServletResponse response, int error, String message, Exception e) throws IOException {
		StringBuilder finalMessage = new StringBuilder(message);
		finalMessage.append(" Cause: ");
		finalMessage.append(e.toString());
		finalMessage.append(". ");
		finalMessage.append(CHECK_THE_LOGS_MESAGE);
		response.sendError(error, finalMessage.toString());
	}
	
	public void destroy() {
		this.controller.getWebApplication().getPluginManager().dispose();
	}
}
