package org.vraptor.plugin.groovy;

import groovy.lang.GroovyClassLoader;

import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.vraptor.VRaptorException;
import org.vraptor.annotations.Component;
import org.vraptor.component.ComponentManager;
import org.vraptor.component.ComponentType;

/**
 * An instance of this class encapsulates a {@link GroovyClassLoader} that it
 * reloads dynamically when the underlying groovy classes have been modified.
 * <p>
 * A base directory, where Groovy classes are expected, is scanned recursively
 * for changes.
 * 
 * @author yduchesne
 * 
 */
public class GroovyLoader implements Runnable {

    private static final Logger LOG = Logger.getLogger(GroovyLoader.class);

    private GroovyClassLoader loader;
    private final ComponentManager manager;
    private final File groovyBaseDir;
    private final long interval;
    private final Map<String, GroovyClassInfo> classes = new HashMap<String, GroovyClassInfo>();

    // / inner class ///
    static class GroovyClassInfo {
        File classFile;
        Class<?> clazz;
        String className;
        long timestamp;

        void touch() {
            timestamp = classFile.lastModified();
        }

        @Override
        public String toString() {
            return "GroovyClassInfo: " + classFile.getName() + ", " + className;
        }
    }

    /**
     * @param manager
     *            the {@link ComponentManager} with which the
     *            {@link ComponentType}s corresponding to groovy classes are
     *            eventually registered.
     * @param groovyBaseDir
     *            the directory to recursively scan.
     * @param interval
     *            the interval (in millis) at which scanning should be
     *            performed.
     * @throws MalformedURLException
     */
    GroovyLoader(ComponentManager manager, File groovyBaseDir, long interval) throws MalformedURLException {

        if (groovyBaseDir == null || !groovyBaseDir.exists()) {
            throw new IllegalStateException("Groovy base dir does not exist.");
        }

        this.manager = manager;
        this.groovyBaseDir = groovyBaseDir;
        this.interval = interval;

    }

    public void run() {
        while (true) {
            try {
                Thread.sleep(interval);
                if (groovyBaseDir != null && groovyBaseDir.exists() && groovyBaseDir.isDirectory()) {
                    loadComponentTypes();
                }
            } catch (VRaptorException e) {
                LOG.error("An exception was thrown while loading Groovy classes", e);
            } catch (InterruptedException e) {
                break;
            }
        }
    }

    void loadComponentTypes() throws VRaptorException {
        Set<GroovyClassInfo> sourceSet = new HashSet<GroovyClassInfo>();
        doLoadComponentTypes("", groovyBaseDir, sourceSet);

        if (sourceSet.size() > 0) {

            loader = new GroovyClassLoader();
            try {
                loader.addURL(groovyBaseDir.toURI().toURL());

                for (GroovyClassInfo info : sourceSet) {
                    if (info.clazz == null) {
                        // new class
                        info.clazz = loader.parseClass(info.classFile);
                        info.touch();
                        classes.put(info.className, info);
                    } else {
                        // modified
                        info.clazz = loader.parseClass(info.classFile);
                        LOG.debug("Detected modification of Groovy class: " + info.clazz.getName());
                        info.touch();
                        classes.put(info.className, info);
                    }
                    if (info.clazz.getAnnotation(Component.class) != null) {
                        ComponentType type = manager.getComponentType(info.clazz);
                        manager.register(type);
                    }
                }
            } catch (IOException e) {
                throw new VRaptorException("Could not load Groovy source(s)", e);
            }
        }

    }

    private void doLoadComponentTypes(String parentPackage, File dir, Set<GroovyClassInfo> sourceSet)
            throws VRaptorException {

        File[] subFiles = dir.listFiles();
        for (File subFile : subFiles) {
            if (subFile.isHidden()) {
                continue;
            }

            String currentPackagePrefix = parentPackage == null || parentPackage.trim().length() == 0 ? ""
                    : parentPackage + ".";
            if (subFile.isDirectory()) {
                String childPackage = currentPackagePrefix + subFile.getName();
                doLoadComponentTypes(childPackage, subFile, sourceSet);
            } else {
                String fileName = currentPackagePrefix + subFile.getName();
                String classname = chopExtension(fileName);
                GroovyClassInfo classInfo = classes.get(classname);
                if (classInfo == null) {
                    classInfo = classInfoFrom(subFile, classname);
                    sourceSet.add(classInfo);
                } else if (subFile.lastModified() != classInfo.timestamp) {
                    sourceSet.add(classInfo);
                }
            }
        }
    }

    private String chopExtension(String fileName) {
        int i = fileName.lastIndexOf('.');
        if (i > 0) {
            return fileName.substring(0, i);
        } else {
            return fileName;
        }
    }

    private GroovyClassInfo classInfoFrom(File subFile, String classname) {
        GroovyClassInfo classInfo;
        classInfo = new GroovyClassInfo();
        classInfo.className = classname;
        classInfo.classFile = subFile;
        classInfo.touch();
        return classInfo;
    }
}