package org.vraptor.plugin.groovy;

import java.io.File;
import java.net.MalformedURLException;
import java.util.Map;

import org.apache.log4j.Logger;
import org.vraptor.VRaptorException;
import org.vraptor.plugin.Disposable;
import org.vraptor.plugin.VRaptorPlugin;
import org.vraptor.webapp.WebApplication;

/**
 * This class implements the {@link VRaptorPlugin} interface "on top" of a {@link GroovyClassLoader}.
 * It allows {@link ComponentType}s to be defined as Groovy classes and supports dynamic reloading upon
 * a) the Groovy classes being modified; b) new Groovy classes being detected.
 * <p>
 * Groovy classes are loaded in a separate classloader (a {@link GroovyClassLoader}, which is completely
 * discarded when new Groovy classes are detected, or when existing ones have been modified.
 * <p>
 * This plugin recursively scans the Groovy classes kept under a specified directory (by default,
 * this directory is expected to be <code>WEB-INF/groovy</code>).
 * <p>
 * A <code>GroovyPlugin</code> is configured as follows in the <code>vraptor.xml</code> file:
 * <pre>
 *  &lt;plugin type="org.vraptor.plugin.groovy.GroovyPlugin"&gt;
 *    &lt;!-- optional configuration, defaults to  /WEB-INF/groovy --&gt;
 *    &lt;property name="baseDir">/WEB-INF/groovy&lt;/property&gt;
 *    &lt;!-- optional configuration, defaults to 1000 (millis) --&gt;
 *    &lt;property name="scanInterval">1000&lt;/property&gt     
 *  &lt;/plugin&gt;
 * </pre>
 * As illustrated, the plugin takes to optional configuration properties, providing default values if these
 * are not provided. It should be noted that a <b>scan interval of 0 disables periodic scanning</b> of Groovy
 * classes, thereby turning off the dynamic reload feature. This is appropriate when deploying into production.
 * <p>
 * Finally, note that the plugin will attempt to resolve the base directory of Groovy classes using the
 * {@link ApplicationContext#getRealPath(String)} method (the {@link ApplicationContext} is acquired from
 * the {@link WebApplication} passed to the plugin - see {@link #init(WebApplication)}).
 * 
 * @author yduchesne
 *
 */
public class GroovyPlugin implements VRaptorPlugin, Disposable {

	public static final String BASE_DIR = "baseDir";
	public static final String SCAN_INTERVAL = "scanInterval";

	public static final String DEFAULT_BASE_DIR = "/WEB-INF/groovy";
	public static final long DEFAULT_SCAN_INTERVAL = 1000;

	private static final Logger LOG = Logger.getLogger(GroovyPlugin.class);

	private WebApplication app;
	private Thread loaderThread;
	private Map<String, String> properties;

	public GroovyPlugin(Map<String, String> properties) {
		this.properties = properties;
	}

	public void init(WebApplication application) throws VRaptorException {

		this.app = application;

		String groovyBase = application.getApplicationContext().getRealPath(property(BASE_DIR, DEFAULT_BASE_DIR));
		if (groovyBase == null) {
			LOG.warn("/WEB-INF/groovy directory not found; scanning of Groovy classes will not be performed");
			return;
		}

		File groovyBaseDir = new File(groovyBase);
		if (!groovyBaseDir.exists()) {
			LOG.warn(groovyBaseDir.getAbsolutePath()
					+ " directory not found; scanning of Groovy classes will not be performed");
			return;
		}
		if (!groovyBaseDir.isDirectory()) {
			LOG.warn(groovyBaseDir.getAbsolutePath()
					+ " is not a directory; scanning of Groovy classes will not be performed");
			return;
		}

		LOG.debug("Loading Groovy; scanning classes under: " + groovyBaseDir.getAbsolutePath());
		try {
			long interval = propertyAsLong(SCAN_INTERVAL, DEFAULT_SCAN_INTERVAL);
			GroovyLoader loader = new GroovyLoader(app.getComponentManager(), groovyBaseDir, interval);
			loader.loadComponentTypes();
			if (interval > 0) {
				loaderThread = new Thread(loader, "VRaptorGroovyPlugin-loader");
				loaderThread.setDaemon(true);
				loaderThread.start();
			}
		} catch (MalformedURLException e) {
			throw new VRaptorException("Could not load Groovy scripting engine");
		}

	}

	public void dispose() {
		if (loaderThread != null) {
			while (loaderThread.isAlive()) {
				loaderThread.interrupt();
				try {
					loaderThread.join();
				} catch (InterruptedException e) {
					break;
				}
			}
			loaderThread = null;
		}
	}

	private long propertyAsLong(String name, long defaultVal) {
		String val = property(name, null);
		if (val == null) {
			return defaultVal;
		} else {
			return Long.parseLong(val);
		}
	}

	private String property(String name, String defaultVal) {
		String val = properties.get(name);
		if (val == null) {
			return defaultVal;
		} else {
			return val;
		}
	}

}
