package org.vraptor.plugin.niceurls;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.vraptor.component.DefaultComponentContainer;
import org.vraptor.http.Dispatcher;
import org.vraptor.http.DispatcherFactory;
import org.vraptor.plugin.niceurls.parser.DefaultRoutesParser;
import org.vraptor.plugin.niceurls.parser.FileSystem;
import org.vraptor.plugin.niceurls.parser.RoutesParser;
import org.vraptor.plugin.niceurls.resolver.DefaultURLResolver;
import org.vraptor.plugin.niceurls.resolver.URLData;
import org.vraptor.plugin.niceurls.resolver.URLResolver;
import org.vraptor.plugin.niceurls.resolver.UnknownURLException;

public class NiceURLFilter implements Filter {
	private static final Logger LOG = Logger.getLogger(NiceURLFilter.class);
	private static final long serialVersionUID = 1084551538406918488L;

	protected static final String INIT_PARAM_CONFIG_FILE = "niceurl-config-file";
	private static final String CONFIG_FILE_DEFAULT = "niceurl.routes";
	public static final String URLDATA_ATTRIBUTE = "org.vraptor.plugin.niceurls.VRaptorURLData";

	private Dispatcher dispatcher;
	private ServletContext servletContext;
	private URLResolver urlResolver;

	public NiceURLFilter() {
	}

	public NiceURLFilter(Dispatcher dispatcher, URLResolver urlResolver, ServletContext servletContext) {
		this.servletContext = servletContext;
		this.dispatcher = dispatcher;
		this.urlResolver = urlResolver;
	}

	public void init(FilterConfig fc) throws ServletException {
		LOG.info("Initializing NiceURLFilter");
		this.servletContext = fc.getServletContext();
		NiceURLComponentContainer niceUrlComponentContainer = new NiceURLComponentContainer(
				new DefaultComponentContainer(servletContext));
		dispatcher = new DispatcherFactory().configure(servletContext,	niceUrlComponentContainer);
		
		urlResolver = new DefaultURLResolver();

		readRoutesConfiguration(fc, new ClasspathContextFileSystem());
	}

	void readRoutesConfiguration(FilterConfig filterConfig, FileSystem fs) throws ServletException {
		RoutesParser parser = new DefaultRoutesParser(urlResolver, fs);
		String routesFileName = readProperty(filterConfig, INIT_PARAM_CONFIG_FILE, CONFIG_FILE_DEFAULT);
		
		try {
			parser.parseFile("/" + routesFileName);
		}
		catch (IOException e) {
			throw new ServletException(e);
		}
	}

	public void destroy() {
		dispatcher.destroy();
	}

	public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain)
			throws IOException, ServletException {
		HttpServletRequest request = (HttpServletRequest) servletRequest;
		HttpServletResponse response = (HttpServletResponse) servletResponse;

		if (requestingStaticFile(request)) {
			deferProcessingToContainer(filterChain, request, response);
			return;
		}

		try {
			setupUrlData(request);
			dispatchToVRaptor(request, response);
		}
		catch (UnknownURLException unknownURL) {
			send404(response, unknownURL.getURL());
		}
	}

	private boolean requestingStaticFile(HttpServletRequest request) throws MalformedURLException {
		URL resourceUrl = servletContext.getResource(uriRelativeToContextRoot(request));
		return resourceUrl != null && isAFile(resourceUrl);
	}

	private boolean isAFile(URL resourceUrl) {
		return !resourceUrl.toString().endsWith("/");
	}

	private void deferProcessingToContainer(FilterChain filterChain, HttpServletRequest request, HttpServletResponse response)
			throws IOException, ServletException {
		if (LOG.isDebugEnabled())
			LOG.debug("deferring URI to container: " + request.getRequestURI());

		filterChain.doFilter(request, response);
	}

	private void setupUrlData(HttpServletRequest request) {
		URLData urlData = urlResolver.resolveURL(uriRelativeToContextRoot(request));
		request.setAttribute(NiceURLFilter.URLDATA_ATTRIBUTE, urlData);
	}

	private void dispatchToVRaptor(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
		if (LOG.isDebugEnabled())
			LOG.debug("new request to " + request.getRequestURI());

		dispatcher.dispatch(request, response, servletContext);
	}
	
	private void send404(HttpServletResponse response, String url) throws IOException {
		String message = "Resource not found at " + url;
		LOG.debug(message);
		response.sendError(404, message);
	}

	private String uriRelativeToContextRoot(HttpServletRequest request) {
		return request.getRequestURI().substring(request.getContextPath().length());
	}

	private String readProperty(FilterConfig fc, String key, String defaultValue) {
		String value = fc.getInitParameter(key);
		return (value != null) ? value : defaultValue;
	}
	
	private static class ClasspathContextFileSystem implements FileSystem {
		public InputStream open(String path) throws FileNotFoundException {
			InputStream file = NiceURLFilter.class.getResourceAsStream(path);
			if (file == null)
				throw new FileNotFoundException("NiceURLs routes configuration file not found at " +
						path);
			
			return file;
		}
	}
}
